package com.soss.system.service.impl;

import com.soss.common.enums.CouponCategoryType;
import com.soss.common.enums.CouponState;
import com.soss.common.enums.CouponUserType;
import com.soss.common.exception.ServiceException;
import com.soss.common.utils.DateUtils;
import com.soss.common.utils.StringUtils;
import com.soss.system.domain.*;
import com.soss.system.domain.vo.CouponUserVo;
import com.soss.system.domain.vo.CouponVo;
import com.soss.system.mapper.*;
import com.soss.system.service.ICouponUserService;
import com.soss.system.utils.ArrayUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户领取优惠券记录表 服务实现类
 * </p>
 *
 * @author caiyt
 * @since 2022-07-21
 */
@Service
@Slf4j
public class CouponUserServiceImpl implements ICouponUserService {
    @Autowired
    private CouponUserMapper couponUserMapper;
    @Autowired
    private CouponMapper couponMapper;
    @Autowired
    private CouponRuleMapper couponRuleMapper;
    @Autowired
    private CustomerMapper customerMapper;
    @Autowired
    private ShopMapper shopMapper;
    @Autowired
    private GoodsMapper goodsMapper;
    @Autowired
    private GoodsSkuMapper goodsSkuMapper;

    @Override
    public void giveUserCoupon(String custId, Integer couponId) {
        Customer cust = customerMapper.selectCustomerById(custId);
        Assert.notNull(cust, "用户不存在[id=" + custId + "]");
        Coupon coupon = couponMapper.selectCouponById(couponId);
        Assert.isTrue(coupon != null && Objects.equals(coupon.getState(), CouponState.ONLINE.getState()), "优惠券状态不合法");
        CouponRule couponRule = couponRuleMapper.selectCouponRuleById(coupon.getRuleId());
        Assert.notNull(couponRule, "未查询到匹配的优惠券规则[id=" + coupon.getRuleId() + "]");
        LocalDateTime now = LocalDateTime.now();
        if (couponRule.getUseStartTime().isAfter(now) || couponRule.getUseEndTime().isBefore(now)) {
            throw new ServiceException("优惠券不在有效期内");
        }
        CouponUser couponUser = new CouponUser();
        couponUser.setCustId(custId);
        couponUser.setCustName(cust.getUserName());
        couponUser.setCustPhone(cust.getPhone());
        couponUser.setCouponId(couponId);
        couponUser.setExpiredTime(this.getCouponExpiredTime(couponRule, now));
        couponUser.setReceiveTime(now);
        couponUser.setSource("赠送");
        couponUser.setType(CouponUserType.GIVE.getType());
        couponUser.setCreatedAt(now);
        couponUser.setUpdatedAt(now);
        couponUserMapper.insertCouponUser(couponUser);
    }

    @Override
    @Transactional
    public void receiveCoupon(Long userId, Integer id) {
        CouponUser couponUser = couponUserMapper.getUserCouponById(id);
        Assert.notNull(couponUser, "未查询到匹配记录[id=" + id + "]");
        Assert.isTrue(couponUser.getCustId().equals(userId), "该券与当前用户信息不符");
        LocalDateTime now = LocalDateTime.now();
        Coupon coupon = couponMapper.selectCouponById(couponUser.getCouponId());
        Assert.notNull(coupon, "未查询到匹配的优惠券信息[id=" + couponUser.getCouponId() + "]");
        CouponRule couponRule = couponRuleMapper.selectCouponRuleById(coupon.getRuleId());
        Assert.notNull(couponRule, "未查询到匹配的优惠券规则信息[id=" + coupon.getRuleId() + "]");
        if (couponRule.getReceivableTime() != null && couponRule.getReceivableTime().isAfter(now)) {
            throw new ServiceException("该券还未到可领取的时间：" + DateUtils.dateTimeNow());
        }
        if (couponRule.getUseEndTime().isBefore(now)) {
            throw new ServiceException("该券已失效");
        }
        couponUser.setExpiredTime(this.getCouponExpiredTime(couponRule, now));
        couponUser.setReceiveTime(now);
        couponUser.setUpdatedAt(now);
        couponUserMapper.updateCouponUser(couponUser);
    }

    private LocalDateTime getCouponExpiredTime(CouponRule couponRule, LocalDateTime now) {
        LocalDateTime expiredTime;
        if (couponRule.getRelativeTime() != null && couponRule.getRelativeTime() > 0) {
            LocalDateTime expireComputerStartTime = couponRule.getUseStartTime().isAfter(now) ? couponRule.getUseStartTime() : now;
            expiredTime = DateUtils.addDaysAndGetEnd(expireComputerStartTime, couponRule.getRelativeTime());
            expiredTime = expiredTime.isBefore(couponRule.getUseEndTime()) ? expiredTime : couponRule.getUseEndTime();
        } else {
            expiredTime = couponRule.getUseEndTime();
        }
        return expiredTime;
    }

    /**
     * 查询用户可用优惠券数量
     *
     * @param custId
     * @return
     */
    @Override
    public int getCustAvailableCouponCnt(String custId) {
        return couponUserMapper.getCustAvailableCouponCnt(custId, LocalDateTime.now());
    }

    /**
     * 查询用户可用优惠券列表
     *
     * @param custId
     * @param couponUserId
     * @return
     */
    @Override
    public List<CouponVo> listCustAvailableCoupon(String custId, Integer couponUserId) {
        List<CouponVo> couponVos = couponUserMapper.listCustAvailableCoupon(custId, couponUserId, LocalDateTime.now());
        transferTabIds(couponVos);
        return couponVos;
    }

    private void transferTabIds(List<CouponVo> couponVos) {
        couponVos.forEach(couponVo -> {
            couponVo.setCategoryIds(ArrayUtil.transStrToCodeList(couponVo.getCategoryIdStr()));
            couponVo.setGoodsIds(ArrayUtil.transStrToLongList(couponVo.getGoodsIdStr()));
            couponVo.setProvince(ArrayUtil.transStrToCodeList(couponVo.getProvinceStr()));
            couponVo.setCity(ArrayUtil.transStrToCodeList(couponVo.getCityStr()));
            couponVo.setArea(ArrayUtil.transStrToCodeList(couponVo.getAreaStr()));
            couponVo.setShopIds(ArrayUtil.transStrToLongList(couponVo.getShopIdStr()));
            couponVo.setWeekLimit(ArrayUtil.transStrToIntList(couponVo.getWeekLimitStr()));
        });
    }

    @Override
    public List<CouponVo> listCustCoupons(String custId, Boolean effectiveFlag) {
        List<CouponVo> couponVos = couponUserMapper.listCustCoupons(custId, effectiveFlag);
        transferTabIds(couponVos);
        return couponVos;
    }

    /**
     * 查询用户领取优惠券列表
     *
     * @param couponUser
     * @return
     */
    @Override
    public List<CouponUserVo> selectCouponUserList(CouponUser couponUser) {
        return couponUserMapper.selectCouponUserList(couponUser);
    }

    @Override
    public void resovleCouponFitable(List<CouponVo> couponVos, Order order) {
        /** 可用门店和时段的校验 */
        this.checkShopAndWeeklyFitable(couponVos, order.getShopId());
        /** 可用饮品范围的判断 */
        checkGoodsFitable(couponVos, order.getOrderDetails());
    }

    private void checkGoodsFitable(List<CouponVo> couponVos, List<OrderDetail> orderDetails) {
        Assert.notEmpty(orderDetails, "商品明细还未传递");
        List<Long> skuIds = orderDetails.stream().map(OrderDetail::getSkuId).collect(Collectors.toList());
        Assert.notEmpty(skuIds, "skuId还未传递");
        List<GoodsSku> goodsSkus = goodsSkuMapper.selectSkuListForOrder(skuIds);
        log.info("sku id size is {} and query size is {}", skuIds.size(), goodsSkus.size());
        Assert.isTrue(goodsSkus.size() == skuIds.size(), "未查询到匹配的sku记录");
        List<Long> goodsIds = goodsSkus.stream().map(GoodsSku::getGoodsId).collect(Collectors.toList());
        List<Goods> goods = goodsMapper.selectGoodsByIds(goodsIds);
        Assert.notEmpty(goods, "未查询到匹配的商品信息[id=" + goodsIds + "]");
        // sku的ID和价格字典
        Map<Long, GoodsSku> skuMap = goodsSkus.stream().collect(Collectors.toMap(GoodsSku::getId, Function.identity()));
        // 订单明细设置
        orderDetails.forEach(orderDetail -> {
            GoodsSku sku = skuMap.get(orderDetail.getSkuId());
            BigDecimal skuNum = new BigDecimal(orderDetail.getNum());
            orderDetail.setOriAmount(skuNum.multiply(sku.getPrice()));
            orderDetail.setUnitPrice(sku.getDiscount());
            orderDetail.setAmountShould(skuNum.multiply(sku.getDiscount()));
        });
        // 订单总额
        BigDecimal orderTotalAmount = orderDetails.stream().map(OrderDetail::getAmountShould).reduce(BigDecimal.ZERO, BigDecimal::add);
        couponVos.stream().filter(couponVo -> StringUtils.isEmpty(couponVo.getNotFitableDesc())).forEach(couponVo -> {
            /** 可用饮品范围的判断 */
            List<Long> fitCouponGoodsIdList = goods.stream().filter(good ->
                    (ArrayUtil.isAllAvailable(couponVo.getCategoryIds()) || ArrayUtil.contains(couponVo.getCategoryIds(), good.getCategory()))
                            && (ArrayUtil.isEmpty(couponVo.getGoodsIds()) || ArrayUtil.contains(couponVo.getGoodsIds(), good.getId())))
                    .map(Goods::getId).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(fitCouponGoodsIdList)) {
                if (!ArrayUtil.isAllAvailable(couponVo.getCategoryIds())) {
                    couponVo.setNotFitableDesc("限定品类使用");
                } else {
                    couponVo.setNotFitableDesc("限定商品使用");
                }
                return;
            }
            /** 券价值的判断 */
            if (couponVo.getType().equals(CouponCategoryType.DEDUCTION.getType())) { // 抵扣
                if (couponVo.getPriceLimit() != null && couponVo.getPriceLimit().compareTo(BigDecimal.ZERO) != 0 && couponVo.getPriceLimit().compareTo(orderTotalAmount) > 0) {
                    couponVo.setNotFitableDesc("未达到满减要求");
                }
                couponVo.setCouponAmount(couponVo.getPriceDiscount().min(orderTotalAmount));
            } else {
                List<OrderDetail> fitOrderDetails = orderDetails.stream().filter(orderDetail -> fitCouponGoodsIdList.contains(orderDetail.getGoodsId())).collect(Collectors.toList());
                BigDecimal fitGoodsAmount = fitOrderDetails.stream().map(OrderDetail::getAmountShould).reduce(BigDecimal.ZERO, BigDecimal::add);
                if (couponVo.getPriceLimit() != null && couponVo.getPriceLimit().compareTo(BigDecimal.ZERO) != 0 && couponVo.getPriceLimit().compareTo(fitGoodsAmount) > 0) {
                    if (couponVo.getType().equals(CouponCategoryType.DISCOUNT.getType())) { // 折扣
                        couponVo.setNotFitableDesc("未达到满减要求");
                    } else { // 免单
                        couponVo.setNotFitableDesc("未达到免单要求");
                    }
                }

                this.wrapperCouponAmount(couponVo, fitOrderDetails, fitGoodsAmount);

                Map<Long, BigDecimal> couponAmountMap = couponVo.getCouponAmountMap();
                orderDetails.forEach(orderDetail -> {
                    BigDecimal couponAmount = couponAmountMap.get(orderDetail.getSkuId());
                    orderDetail.setCouponAmount(couponAmount == null ? BigDecimal.ZERO : couponAmount.negate());
                    orderDetail.setRealAmount(orderDetail.getAmountShould().add(orderDetail.getCouponAmount()));
                });
            }
        });
    }

    private void wrapperCouponAmount(CouponVo couponVo, List<OrderDetail> fitOrderDetails, BigDecimal fitGoodsAmount) {
        Map<Long, BigDecimal> couponAmountMap = new HashMap<>();
        if (couponVo.getOrderLimit() != null && couponVo.getOrderLimit()) { // 单杯
            OrderDetail orderDetail = fitOrderDetails.stream().max(Comparator.comparing(OrderDetail::getUnitPrice)).get();
            BigDecimal discountAmount = orderDetail.getUnitPrice();
            if (couponVo.getType().equals(CouponCategoryType.DISCOUNT.getType())) { // 折扣
                discountAmount = this.getDiscountAmount(discountAmount, couponVo.getPriceDiscount());
            }/* else { // 免单
            }*/
            couponVo.setCouponAmount(discountAmount);
            couponAmountMap.put(orderDetail.getSkuId(), couponVo.getCouponAmount());
        } else { // 整单
            if (couponVo.getType().equals(CouponCategoryType.DISCOUNT.getType())) { // 折扣
                couponVo.setCouponAmount(this.getDiscountAmount(fitGoodsAmount, couponVo.getPriceDiscount()));
                fitOrderDetails.forEach(orderDetail -> orderDetail.setCouponAmount(this.getDiscountAmount(orderDetail.getAmountShould(), couponVo.getPriceDiscount())));
                // 订单明细优惠总和
                BigDecimal detailCouponSum = fitOrderDetails.stream().map(OrderDetail::getCouponAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                // 如果订单明细优惠总额大于订单总优惠金额，则将订单明细最后一条记录的优惠金额去掉差额
                if (detailCouponSum.compareTo(couponVo.getCouponAmount()) != 0) {
                    OrderDetail orderDetail = fitOrderDetails.get(fitOrderDetails.size() - 1);
                    orderDetail.setCouponAmount(orderDetail.getCouponAmount().add(couponVo.getCouponAmount().subtract(detailCouponSum)));
                }
            } else { // 免单
                couponVo.setCouponAmount(fitGoodsAmount);
            }
            fitOrderDetails.forEach(orderDetail -> couponAmountMap.put(orderDetail.getSkuId(), orderDetail.getCouponAmount()));
        }
        couponVo.setCouponAmountMap(couponAmountMap);
    }

    private void checkShopAndWeeklyFitable(List<CouponVo> couponVos, Long shopId) {
        Assert.notNull(shopId, "商店id还未传递");
        Shop shop = shopMapper.selectShopById(shopId);
        Assert.notNull(shop, "未查询到匹配的商店[id=" + shopId + "]");
        // 当天周几
        int dayOfWeek = Calendar.getInstance().get(Calendar.DAY_OF_WEEK);
        couponVos.forEach(couponVo -> {
            /** 可用门店的判断 */
            if (!ArrayUtil.isAllAvailable(couponVo.getProvince()) && ArrayUtil.hasContents(couponVo.getProvince())
                    && !ArrayUtil.contains(couponVo.getProvince(), shop.getProvince())) {
                couponVo.setNotFitableDesc("限定区域使用");
                return;
            }
            if (!ArrayUtil.isEmpty(couponVo.getCity()) && ArrayUtil.hasContents(couponVo.getCity()) && !ArrayUtil.contains(couponVo.getCity(), shop.getCity())) {
                couponVo.setNotFitableDesc("限定区域使用");
                return;
            }
            if (!ArrayUtil.isEmpty(couponVo.getArea()) && ArrayUtil.hasContents(couponVo.getArea()) && !ArrayUtil.contains(couponVo.getArea(), shop.getZone())) {
                couponVo.setNotFitableDesc("限定区域使用");
                return;
            }
            if (!ArrayUtil.isEmpty(couponVo.getShopIds()) && !ArrayUtil.contains(couponVo.getShopIds(), shop.getId())) {
                couponVo.setNotFitableDesc("限定门店使用");
                return;
            }
            /** 可用时段的判断 */
            if (!ArrayUtil.isAllAvailable(couponVo.getWeekLimit()) && !ArrayUtil.contains(couponVo.getWeekLimit(), dayOfWeek)) {
                couponVo.setNotFitableDesc("限定时段使用");
            }
        });
    }

    private BigDecimal getDiscountAmount(BigDecimal amount, BigDecimal priceDiscount) {
        // 打折后的价格
        BigDecimal priceAfterDiscount = amount.multiply(priceDiscount.divide(BigDecimal.TEN, 4, BigDecimal.ROUND_HALF_UP)).setScale(2, BigDecimal.ROUND_HALF_UP);
        // 跟原价比取最小值
        priceAfterDiscount = priceAfterDiscount.min(amount);
        // 获取优惠金额
        return amount.subtract(priceAfterDiscount);
    }
}
